/*************************************************************************************
 
LMS Algorithms

Author: Eric Breimer
File: lmsalgo.h
Description:  Algorithm for finding the longest monotonically increasing sequence of 
              n points in d dimensions. 

*************************************************************************************/
#ifndef LMSALGO_H
#define LMSALGO_H

#include <vector>
#include <list>
#include <algorithm>
#include <cassert>

using std::vector;
using std::list;

/*************************************************************************************
 
*************************************************************************************/
double Vd(double r, int d) {
  double rd = (double) pow(r,d);
  double temp = (double) d*rd;
  double output = (double) temp/r;
  output = (double) output - temp;
  output = (double) rd + output;
  return output;
}  

/*************************************************************************************
 
*************************************************************************************/
double F(const double l, const int d, const double r) {
  double output;
  double thresh = (double) 1 - r;
  double vd = Vd(r,d);
  double rtd = (double) sqrt(d);
  thresh = rtd * thresh;
  if (l <= thresh) {
    output = (double) pow(r,d-1);
    output = (double) output*l;
    output = (double) output*rtd;
    output = (double) output/vd;
  }
  else {
    output = (double) l/rtd;
    output = (double) 1 - output;
    output = (double) pow(output,d);
    output = (double) output/vd;
    output = (double) 1 - output;
  }
  return output;
}

/*************************************************************************************
 
*************************************************************************************/
double F1(const double u, const int d, const double r) {
  double temp;
  double output;
  double thresh;
  double rd =  (double) pow(r,d);  
  double rd1 = (double) pow(r,d-1);
  double vd =  (double) Vd(r,d);
  double rtd = (double) sqrt(d);
  double d1 =  (double) 1/d;  
  thresh = (double) rd/vd;
  thresh = (double) 1 - thresh;
  if (u <= thresh) {
    temp = (double) vd*u;
    output = (double) rd1*rtd;
    output = (double) temp/output;
  }
  else {
    temp = (double) 1-u;
    temp = (double) vd*temp;
    output = (double) pow(temp,d1);
    output = (double) 1 - output;
    output = (double) rtd*output;
    //output = (double) rtd*((double) 1 - pow(vd*((double) 1-u), d1));
  }
  return output;
}
    
/*************************************************************************************
 
*************************************************************************************/
void G(vector<double> & p, double & l, double & t, int & k, const int n, const int d, const double r) {
  int x;
  int idx = (int) ceil(d*drand48())-1;
  double temp;
  double input;
  double output;
  double u = (double) drand48();
  double f = (double) F(l,d,r); 
  temp = (double) 1 - u;
  input = (double) 1/(n-k+1);
  input = (double) pow(temp,input);
  temp = (double) 1 - f;
  input = (double) temp*input;
  input = (double) 1 - input;
  

  //input = (double) 1 - ((double) 1 - f)*pow((double) 1 - u,(double) 1/(n-k+1));
  output = (double) F1(input,d,r);
  l = (double) output;
  t = (double) l/sqrt(d);
  k++;
  for (x = 0; x < d; x++)  
    if (x == idx)
      p[x] = (double) t;
    else if (t < (1-r)) {
      temp = (double) r*drand48();
      p[x] = (double) t + temp;
      //p[x] = (double) t + r*random_(reseed);
    }
    else {
      temp = (double) 1 - t;
      temp = (double) temp*drand48();
      p[x] = (double) t + temp;
      //p[x] = (double) t + ((double)1-t)*random_(reseed);
    }
  p[d] = (double) 1;
}

/*************************************************************************************
 
*************************************************************************************/
bool above(vector<double> & z, vector<double> & w, int d) {
  int x;
  for (x = 0; x < d; x++) 
    if (z[x] <= w[x])
      return (bool) 0;
  return (bool) 1;
}

/*************************************************************************************
 
*************************************************************************************/
bool below(vector<double> & w, double t, int d) {
  int x;
  for (x = 0; x < d; x++) 
    if (w[x] > t)
      return (bool) 0;
  return (bool) 1;
}


/*************************************************************************************
 
Find the longest monotonically increasing subsequence (lms)

*************************************************************************************/
int lms(int n, int d, double r) {
  vector<double> point(d+1);
  list<vector<double> > ws;
  list<vector<double> >::iterator i, i_del;
  double l = 0;
  double t = 0;
  int k = 1;
  int lms_optimal = 0;
  int lms_barrier = 0;
  int lms = 0;
  int x;

  G(point,l,t,k,n,d,r);
  ws.push_back(point);
  for (x = 1; x < n; x++) {
    G(point,l,t,k,n,d,r);
    lms = lms_barrier;  
    i = ws.begin(); 
    while (i != ws.end()) {
      if (above(point,*i,d)&&(((int)(*i)[d]) > lms)) 
	lms = ((int) (*i)[d]);
      i_del = i;
      i++;
      if (below(*i_del,t,d)) {
	if (((int) (*i_del)[d]) > lms_barrier)
	  lms_barrier = ((int) (*i_del)[d]);
	ws.erase(i_del);
      }
    }
    if ((lms+1) > lms_optimal) 
      lms_optimal = lms+1;	
    point[d] = (double) lms+1;
    ws.push_back(point);
  }
  return lms_optimal;
}
#endif
