/*************************************************************************************
 
LMS Misc Functions

Author: Eric Breimer
File: lmsmisc.h

Description:  Misc. functions used by the LMS Program.  Includes functions for writing
and reading from files, computing averages, maximums, minimums, standard deviations,
and printing vectors.

*************************************************************************************/
#ifndef LMSMISC_H
#define LMSMISC_H

#include <vector>
#include <fstream>

using std::vector;
using std::cout;
using std::cerr;
using std::endl;
using std::ofstream;

/*************************************************************************************
 
Write Input

Description:  Writes the input points to a file (one point per line, each corrdinate is
              separated by a tab).
Input: input file, length n, dimension d, and input vector v
Output: file output

*************************************************************************************/
void write(char * name, int n, int d, vector<vector<double> > v) {
  int x, y;
  ofstream out;
  out.open(name);
  if(out == NULL) {
    cerr << "Failure to open file: " << name << endl;
    return;
  }     
  for (x = 0; x < n; x++) {
    for (y = 0; y < d; y++) {
      out << v[x][y] << "\t";
    }
    out << endl;
  } 
  out.close();
}

/*************************************************************************************
 
Write Solution

Description:  Writes the solution points to a file (one point per line, each 
              corrdinate is separated by a tab).
Input: input file, solution vector s, dimension d, and input vector v
Output: file output

*************************************************************************************/
void write(char * name, vector<int> s, int d, vector<vector<double> > v) {
  int x, y;
  ofstream out; 
  out.open(name);
  if(out == NULL) {
    cerr << "Failure to open file: " << name << endl;
    return;
  }     
  for (x = 0; x < s.size(); x++) {
    for (y = 0; y < d; y++) {
      out << v[s[x]][y] << "\t";
    }
    out << endl;
  } 
  out.close();
}

/*************************************************************************************
 
Write Data

Description: Writes a vector to a file.
Input: file name, vector
Output: file output

*************************************************************************************/
template <class DataType>
void write(char * name, vector<DataType> & v) {
  int x;
  std::ofstream out;
  out.open(name);
  if(out == NULL) {
    std::cerr << "Failure to open file: " << name << "\n";
    return;
  }     
  for (x = 0; x < v.size(); x++) {
    out << x << "\t" << v[x] << "\n";
  } 
  out.close();
}

/*************************************************************************************
 
Write Selected Data

Description: Given a vector of indices s and a vector of values v, writes v[si]
             for all indices si from s. 
Input: file name, vector s, vector v
Output: file output

*************************************************************************************/
template <class SelectedDataType>
void write(char * name, vector<int> & s, vector<SelectedDataType> & v) {
  int x;
  std::ofstream out;
  out.open(name);
  if(out == NULL) {
    std::cerr << "Failure to open file: " << name << "\n";
    return;
  }     
  for (x = 0; x < s.size(); x++) {
    out << s[x] << "\t" << v[s[x]] << "\n";
  } 
  out.close();
}

/*************************************************************************************
 
Print vector

Description:  Prints a vector delimited by spaces
Input: vector of integers
Output: cout

*************************************************************************************/
template <class VectorType>
void print_vector(vector<VectorType> & v) {
  int x = 0;
  for (x = 0; x < v.size(); x++) {
    cout << v[x] << " ";
  }
  cout << endl;
}

/*************************************************************************************
 
Selected Max

Description:  Given the solution indices and input points, computes the maximum 
              among only the solution indices.
Input: solution vector s, input vector v
Output: in

*************************************************************************************/
int selectedMax(vector<int> & s, vector<int> & v) {
  int x;
  int max = v[s[1]];
  for (x = 2; x < s.size(); x++) {
    if (v[s[x]] > max) {
      max = v[s[x]];
    }
  }
  return max;
}

/*************************************************************************************
 
Max Radius

Description:  Given the solution indices and input points, computes the maximum 
              radius among only the solution points
Input: dimension d, solution vector s, input vector p
Output: double

*************************************************************************************/
double maxrad(int d, vector<int> & s, vector<vector<double> > & p) {
  double ave = 0;
  double sum = 0;
  double max = 0;
  int x, y;
  for (x = 0; x < s.size(); x++) {
    ave = 0;
    for (y = 0; y < d; y++) {
      ave = ave + p[s[x]][y];
    }
    ave /= d;
    sum = 0;
    for (y = 0; y < d; y++) {
      sum = sum + pow(p[s[x]][y]-ave,2);
    }
    sum = sqrt(sum);
    if (sum > max)
      max = sum;
  }
  return max;
}

/*************************************************************************************
 
radius

Description:  Given a point (vector<double>) in d dimension, computes the radius of 
              the point.
Input: vector v
Output: double

*************************************************************************************/
double radius(int d, vector<double> & p) {
  double ave = 0;
  double sum = 0;
  int x;
  for (x = 0; x < d; x++) {
    ave = ave + p[x];
  }
  ave /= d;
  for (x = 0; x < d; x++) {
    sum = sum + pow(p[x]-ave,2);
  }
  sum = sqrt(sum);
  return sum;
}

/*************************************************************************************
 
Compute Average

Description:  Given a vector (int or double), computes the average of the vector
Input: vector v
Output: average value (double)

*************************************************************************************/
template <class AveType>
double computeAve(vector<AveType> & v) {
  int x;
  double ave = 0;
  for (x = 0; x < v.size(); x++) {
    ave += v[x];
  }
  ave /= v.size();
  return ave;
}

/*************************************************************************************
 
Compute Standard Deviation

Description:  Given a vector (int or double) and an average, computes the 
              standard deviation
Input: vector v, precomputed average value
Output: standard deviation

*************************************************************************************/
template <class DevType>
double computeDev(vector<DevType> & v, double ave) {
  int x;
  double dev = 0;
  for (x = 0; x < v.size(); x++) {
    dev += pow(v[x] - ave, 2);
  }
  dev /= v.size();
  dev = sqrt(dev);
  return dev;
}

/*************************************************************************************
 
Compute Max

Description:  Given a vector, computes the maximum of the vector
Input: vector v
Output: maximum value

*************************************************************************************/
template <class MaxType>
MaxType computeMax(vector<MaxType> & v) {
  int x;
  MaxType max = v[0];
  for (x = 1; x < v.size(); x++) {
    if (v[x] > max) {
      max = v[x];
    }
  }
  return max;
}
   
/*************************************************************************************
 
Compute Min

Description:  Given a vector, computes the minimum of the vector
Input: vector v
Output: minimum value

*************************************************************************************/
template <class MinType>
MinType computeMin(vector<MinType> & v) {
  int x;
  MinType min = v[0];
  for (x = 1; x < v.size(); x++) {
    if (v[x] < min) {
      min = v[x];
    }
  }
  return min;
}

double computeNd(int n, int d, double r) {

  double d1 = (double) 1/d;
  double top = (double) pow(n, d1);
  double bot = (double) d/r;
  bot = (double) bot - d;
  bot = (double) bot + 1;
  bot = (double) pow(bot, d1);
  bot = (double) r*bot;
  return (double) top/bot;
}  

#endif
